var GraphEventManager = (function (){
    'use strict';
    /**
     * @description Manage all of the graph event and some more.
     * @param {Graph} graph
     * @typedef GraphEventManager Manage all of the graph event and some more.
     */
    function GraphEventManager(graph) {
        /**
         * @description used to get cy instance.
         * @type Graph
         */
        var graph_ = graph;

        /**
         * @description When the user apply his graph settings (layout, group, filter).
         */
        function grapheActions() {
            $("#graphe_action__button").click(function () {
                if (!allFilterActivated()) {
                    var layout = JSON.parse($("#layout-select").val());
                    if (typeof Storage !== "undefined") sessionStorage.setItem("layout", JSON.stringify(layout));

                    changeGraphElement($("input[name=\"group\"]:checked").val());

                    var filter_status = $("input[name=\"activate_deactivate_classesfilter\"]:checked").val();
                    if (typeof Storage !== "undefined") sessionStorage.setItem("filter_status", filter_status);
                    if (filter_status === "on") {
                        filterGrapheElement();
                    } else {
                        graph_.endFilter();
                    }

                    if (layout.type === "cyto") {
                        changeCytoLayout(layout);
                    } else {
                        changeVizLayout(layout);
                    }


                    graph_.run();
                } else {
                    alert("All nodes are filtered out. Please un-filter at least one class.");
                }
            });
        }
        /**
         * @description Apply filter settings.
         */
        function filterGrapheElement() {
            if ($(".nodeclass:checked").length > 0) {
                var filter_arr = getLabelFiltre();
                if (filter_arr.length > 0) {
                    if (typeof(Storage) !== "undefined") sessionStorage.setItem("graphe_filtre", filter_arr.join(","));
                    graph_.startFilter(filter_arr);
                } else {
                    if (typeof(Storage) !== "undefined") sessionStorage.setItem("graphe_filtre", []);
                    graph_.endFilter();
                }
            }
        }
        /**
         * @description Apply the group settings.
         * @param {string} wanted_group
         */
        function changeGraphElement(wanted_group) {
            if (typeof(Storage) !== "undefined") sessionStorage.setItem("layout_group", wanted_group);

            switch (wanted_group) {
                case "nogroup":
                    wanted_group = utilisation.origin;
                    break;
                case "groupbycause":
                    wanted_group = utilisation.cause;
                    break;
                case "groupbyimpact":
                    wanted_group = utilisation.impact;
                    break;
                default:
                    wanted_group = "";
                    break;
            }
            graph_.changeElement(wanted_group);
        }
        /**
         * @description Changement de layout spécifique à un changementGrapgViz.
         * @param {Object} layout
         */
        function changeVizLayout(layout) { graph_.changeLayout(layout); }

        /**
         * @description Changement de layout spécifique à un layout cy.
         * @param {Object} layout
         */
        function changeCytoLayout(layout) { graph_.changeLayout(layout); }

        /**
         * @description Gestion du resizing de lafenetre.
         */
        function resizing() { $(window).resize(function () { graph_.resize(); }); }

        /**
         * @description Gestion de la selection d'un node par l'utilisateur.
         * Changement graphique pour une meilleur vision du node et ce qui y est lié.
         * @param {Object} cy Correspond à une instance de cy
         */
        function selectionNode(cy) {
            cy.nodes().on("select", function (evt) {
                var node = evt.cyTarget, ok_selection = true, jsonNode = node.data(node_a.data.hiddenProperties)[node_p.me];

                if (node.isParent()) {
                    node.unselect();
                    graph_.majGroup(node, true);
                } else if (!node.hasClass("collapse_group")) {
                    if ((node.isChild() && !node.scratch("selected") || !node.isChild()))
                        if (jsonNode.areYou(node_p.impacted) || ((jsonNode.areYou(node_p.causeOf) && jsonNode.getProp(node_p.causeOf).length !== 0))) {
                            node.unselect();
                            cannotselecttwoonsamepath();
                        } else {
                            /*
                             * Empêche l'éxecution si l'évènement est déclenché plusieurs fois.
                             * Problème Cytoscape qui déclenche plusieurs fois l'évènement sur un child node.
                             */
                            if (node.isChild()) node.scratch("selected", true);
                            graph_.majNode(node, true);
                        }
                } else { /* Must be a node with "collapse_group" class, so a group node. */
                    node.unselect();
                    cannotselectgroup();
                }
            });
        }
        /**
         * @description Lors de la désélection d'un node, il faut le faire repassé à l'état initiale ainsi que tous ses impacts.
         * @param {Object} cy Correspond à une instance de cy
         */
        function deselectionNode(cy) {
            cy.nodes().on("unselect", function (evt) {
                if (!$("#cannotselect").hasClass("hidden")) $("#cannotselect").addClass("hidden");
                var node = evt.cyTarget;

                if (!node.isParent() && !node.hasClass("collapse_group")) {
                    /* Vérification de la sélection du node.
                     * Problème de Cytoscape survenant lorsque le node est child, Cytoscape déclenche plusieurs fois l'évènement pour le même node.
                     */
                    if (node.data(node_a.data.hiddenProperties)[node_p.me].areYou(node_p.selected)) {
                        if (node.isChild()) node.scratch("selected", false);
                        graph_.majNode(node, false);
                    }
                }
            });
        }

        /**
         * @description Add context to the different elements.
         * @param {object} cy Cytoscape instance.
         */
        function context (cy) {
            cy.on("cxttap", function (evt) {
                var mouseEvent = evt.originalEvent;

                $(window).contextMenu("menu", graph_.getContext(evt), {
                    triggerOn: "contextmenu"
                })
                .contextMenu("open", {
                    top: mouseEvent.clientY,
                    left: mouseEvent.clientX
                });
            });
        }

        /**
         * @description Initialisation des évènement cy.
         */
        this.initEventCy = function (cy) {
            selectionNode(cy); deselectionNode(cy); context(cy);
        };

        /**
         * @description Réduit ou étend les groupes selon la demande utiisateur.
         */
        function collapse_group () {
            $("input[name='collapse_group']").change(function () {
                if (typeof(Storage) !== "undefined") sessionStorage.setItem("collapse_group", $(this).val());

                graph_.setCollapse($("input[name='collapse_group']:checked").val());
            });
        }

        grapheActions();
        resizing();
        collapse_group();
    }

    return GraphEventManager;
}());